package betterwithmods.module.hardcore.creatures;

import betterwithmods.common.items.ItemArcaneScroll;
import betterwithmods.module.Feature;
import betterwithmods.util.WorldUtils;
import com.google.common.collect.Maps;
import net.minecraft.block.BlockPumpkin;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.entity.boss.EntityWither;
import net.minecraft.entity.monster.*;
import net.minecraft.entity.passive.EntityBat;
import net.minecraft.entity.passive.EntitySquid;
import net.minecraft.init.Enchantments;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.*;
import net.minecraft.world.DimensionType;
import net.minecraftforge.event.entity.living.LivingDropsEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import java.util.HashMap;
import java.util.function.Predicate;

public class HCEnchanting extends Feature {
    private static boolean steelRequiresInfernal;

    public static boolean canEnchantSteel() {
        return !steelRequiresInfernal;
    }

    @Override
    public void setupConfig() {
        steelRequiresInfernal = loadPropBool("Steel Requires Infernal Enchanter", "Soulforged Steel tools can only be enchanted with the Infernal Enchanter", true);
    }

    @Override
    public String getFeatureDescription() {
        return "Adds Arcane Scroll drops to specific mobs, used for enchanting with the Infernal Enchanter";
    }

    @FunctionalInterface
    public interface ScrollDrop {
        ItemStack getScroll(EntityLivingBase entity);

        default double getChance() {
            return 0.1;
        }
    }

    private static final HashMap<Class<? extends EntityLivingBase>, ScrollDrop> SCROLL_DROPS = Maps.newHashMap();

    public static void addScrollDrop(Class<? extends EntityLivingBase> clazz, Enchantment enchantment) {
        addScrollDrop(clazz, (entity) -> ItemArcaneScroll.getScrollWithEnchant(enchantment));
    }

    public static void addScrollDrop(Class<? extends EntityLivingBase> clazz, ItemStack scroll) {
        addScrollDrop(clazz, (entity) -> scroll);
    }

    public static void addScrollDrop(Class<? extends EntityLivingBase> clazz, ScrollDrop scroll) {
        SCROLL_DROPS.put(clazz, scroll);
    }


    @Override
    public void init(FMLInitializationEvent event) {
        addScrollDrop(EntitySlime.class, Enchantments.field_180310_c);
        addScrollDrop(EntityPigZombie.class, Enchantments.field_77329_d);
        addScrollDrop(EntityBat.class, Enchantments.field_180309_e);
        addScrollDrop(EntityCreeper.class, Enchantments.field_185297_d);
        addScrollDrop(AbstractSkeleton.class, (entity) -> {
            if (entity.field_70170_p.field_73011_w.func_186058_p() == DimensionType.NETHER)
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_185312_x);
            else
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_180308_g);
        });
        addScrollDrop(EntitySquid.class, Enchantments.field_185298_f);
        addScrollDrop(EntityWitch.class, Enchantments.field_185299_g);
        addScrollDrop(EntityZombie.class, Enchantments.field_185303_l);
        addScrollDrop(EntitySpider.class, Enchantments.field_180312_n);
        addScrollDrop(EntityMagmaCube.class, Enchantments.field_77334_n);
        addScrollDrop(EntityEnderman.class, Enchantments.field_185306_r);
        addScrollDrop(EntityGhast.class, Enchantments.field_185310_v);
        addScrollDrop(EntityBlaze.class, Enchantments.field_185311_w);
        addScrollDrop(EntityShulker.class, new ScrollDrop() {
            @Override
            public ItemStack getScroll(EntityLivingBase entity) {
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_185296_A);
            }

            @Override
            public double getChance() {
                return 0.001;
            }
        });
        addScrollDrop(EntityDragon.class, new ScrollDrop() {
            @Override
            public ItemStack getScroll(EntityLivingBase entity) {
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_191530_r);
            }

            //Always drops
            @Override
            public double getChance() {
                return 1;
            }
        });
        addScrollDrop(EntityWither.class, new ScrollDrop() {
            @Override
            public ItemStack getScroll(EntityLivingBase entity) {
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_180313_o);
            }

            //Always drops
            @Override
            public double getChance() {
                return 1;
            }
        });
        addScrollDrop(EntitySilverfish.class, entity -> {
            if (entity.field_70170_p.field_73011_w.func_186058_p() == DimensionType.THE_END) {
                return ItemArcaneScroll.getScrollWithEnchant(Enchantments.field_185305_q);
            }
            return null;
        });

        //TODO
        // SHARPNESS -> Butcher Trade
        // LOOTING -> Farmer Trade
        // UNBREAKING -> Blacksmith Trade
        // FORTUNE ->  Priest Trade
        // POWER   -> Librarian Trade
        // LUCK_OF_THE_SEA, LURE -> Fisherman Trade
        //TODO new things
        /*
        DEPTH_STRIDER Guardian
        FROST_WALKER Polar Bear
        BINDING_CURSE illager
        SWEEPING Wither Skeleton?
        MENDING REMOVE
        VANISHING_CURSE illager
         */
    }

    @Override
    public boolean hasSubscriptions() {
        return true;
    }

    @SubscribeEvent
    public void onDeath(LivingDropsEvent event) {
        for (Class<? extends EntityLivingBase> entity : SCROLL_DROPS.keySet()) {
            if (entity.isAssignableFrom(event.getEntityLiving().getClass())) {
                ScrollDrop drop = SCROLL_DROPS.get(entity);
                if (drop.getScroll(event.getEntityLiving()) != null) {
                    if (event.getEntityLiving().func_70681_au().nextDouble() <= drop.getChance()) {
                        WorldUtils.addDrop(event, drop.getScroll(event.getEntityLiving()));
                    }
                }
            }
        }
    }


    public enum InfernalEnchantmentType {
        ALL(item -> false),
        ARMOR(item -> item instanceof ItemArmor),
        ARMOR_FEET(item -> item instanceof ItemArmor && ((ItemArmor) item).field_77881_a == EntityEquipmentSlot.FEET),
        ARMOR_LEGS(item -> item instanceof ItemArmor && ((ItemArmor) item).field_77881_a == EntityEquipmentSlot.LEGS),
        ARMOR_CHEST(item -> item instanceof ItemArmor && ((ItemArmor) item).field_77881_a == EntityEquipmentSlot.CHEST),
        ARMOR_HEAD(item -> item instanceof ItemArmor && ((ItemArmor) item).field_77881_a == EntityEquipmentSlot.HEAD),
        WEAPON(item -> item instanceof ItemSword),
        TOOL(item -> item instanceof ItemTool),
        FISHING_ROD(item -> item instanceof ItemFishingRod),
        BREAKABLE(Item::func_77645_m),
        BOW(item -> item instanceof ItemBow),
        WEARABLE(item -> item instanceof ItemArmor || item instanceof ItemElytra || item instanceof ItemSkull || (item instanceof ItemBlock && ((ItemBlock) item).func_179223_d() instanceof BlockPumpkin));

        private Predicate<Item> delegate = null;

        InfernalEnchantmentType(Predicate<Item> delegate) {
            this.delegate = delegate;
        }

        public boolean canEnchantItem(Item item) {
            return this.delegate != null && this.delegate.test(item);
        }

        public static InfernalEnchantmentType[] VALUES = values();

        public static InfernalEnchantmentType fromEnchantment(Enchantment enchantment) {
            if (enchantment.field_77351_y != null) {
                return VALUES[enchantment.field_77351_y.ordinal()];
            }
            return ALL;
        }
    }
}
